#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <qwidget.h>
#include <qstring.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qlistview.h>
#include <qbrush.h>
#include <qsizepolicy.h>
#include <qsize.h>
#include <jack/jack.h>
#include "connector.h"
#include "jackport.h"
#include "jackclient.h"
#include "jackgui.h"

Connector::Connector(QObject *p_jackGui, QWidget* parent, const char *name) 
                     : QWidget (parent, name) {
  
  jackGui = p_jackGui;
  QObject::connect(((JackGui *)jackGui)->readPorts, SIGNAL(contentsMoving(int, int)),
                   this, SLOT(move(int, int)));
  QObject::connect(((JackGui *)jackGui)->writePorts, SIGNAL(contentsMoving(int, int)),      
                   this, SLOT(move(int, int)));       
}

void Connector::arrow(int x1, int y1, int x2, int y2) {

    QPainter p(this);
    float x, y, r, phi, psi;
 
    p.drawLine(x1, y1, x2, y2);
    r = 16.0;
    phi = 0.3 * M_PI_4;
    psi = asin((float(x2) - float(x1)) / hypot(float(x2-x1), float(y2-y1)));
    psi = (y2 > y1) ? psi : M_PI - psi;
    x = float(x2) - r * sin(psi-phi);
    y = float(y2) - r * cos(psi-phi);
    p.drawLine(x2, y2, int(x), int(y));
    x = float(x2) - r * sin(psi+phi);
    y = float(y2) - r * cos(psi+phi);
    p.drawLine(x2, y2, int(x), int(y));
}

void Connector::paintEvent(QPaintEvent *) {

    int rpos, wpos, tmp;
    int delta, width;
    int l1, l2, l3;
    PortItem *r_listViewItem, *w_listViewItem;

    width = this->width();
    for (l1 = 0; l1 < ((JackGui *)jackGui)->readClientList.count(); l1++) {
      for (l2 = 0; l2 < ((JackGui *)jackGui)->readClientList.at(l1)->portList.count(); l2++) {
        r_listViewItem = ((JackGui *)jackGui)->readClientList.at(l1)->portList.at(l2)->listViewItem;
        if (r_listViewItem->isVisible()) {
          tmp = ((JackGui *)jackGui)->readClientList.at(l1)->portList.at(l2)->connectedPorts.count();
          for (l3 = 0; l3 < ((JackGui *)jackGui)->readClientList.at(l1)->portList.at(l2)->connectedPorts.count(); l3++) {
            w_listViewItem = ((JackPort *)((JackGui *)jackGui)->readClientList.at(l1)->portList.at(l2)->connectedPorts.at(l3))->listViewItem;
            if (w_listViewItem->isVisible()) {
              rpos = ((JackGui *)jackGui)->readPorts->itemRect(r_listViewItem).bottom();
              wpos = ((JackGui *)jackGui)->writePorts->itemRect(w_listViewItem).bottom();
              delta = r_listViewItem->height();
              if ((rpos > 0) && (wpos > 0)) {
                arrow(0, rpos + delta, width, wpos + delta);
              }
            }
          }
        }
      }
    }    
}

void Connector::updateConnector() {

  repaint(true);
}

void Connector::move(int x, int y) {

    update();
}

QSize Connector::sizeHint() const {

  return QSize(CONNECTOR_MINIMUM_WIDTH, CONNECTOR_MINIMUM_HEIGHT);
}

QSizePolicy Connector::sizePolicy() const {

  return QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
}

void Connector::resizeEvent (QResizeEvent* )
{
  repaint(true);
}
