#include <stdio.h>
#include <stdlib.h>
#include <qthread.h>
#include <qwidget.h>
#include <qstring.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qframe.h>
#include <qheader.h>
#include <qpointarray.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qmainwindow.h>
#include <qlistbox.h>
#include <jack/jack.h>
#include "connector.h"
#include "jackgui.h"
#include "jackport.h"
#include "jackclient.h"
#include "portitem.h"

JackGui::JackGui(jack_client_t *p_jack_handle, QWidget *parent, const char *name) : QVBox(parent, name) {

  jack_handle = p_jack_handle;
  setSpacing(10);
  aboutWidget = new QMessageBox(this); 
  QHBox *portConnectorBox = new QHBox(this);
  portConnectorBox->setMargin(10);
  readPorts = new QListView(portConnectorBox);
  readPorts->setRootIsDecorated(true);
  readPorts->header()->setClickEnabled(false);
  readPorts->setAllColumnsShowFocus(true);
  readPorts->addColumn("Readable Clients / Ports");
  QHBox *connectorBox = new QHBox(portConnectorBox);
  writePorts = new QListView(portConnectorBox);
  writePorts->setRootIsDecorated(true);
  writePorts->header()->setClickEnabled(false);
  writePorts->setAllColumnsShowFocus(true);
  writePorts->addColumn("Writeable Clients / Ports");
  readClientList.setAutoDelete(true);
  writeClientList.setAutoDelete(true);
  connector = new Connector(this, connectorBox);  
  QHBox *buttonBox = new QHBox(this);  
  buttonBox->setMargin(5);
  new QWidget(buttonBox);
  QPushButton *connectButton = new QPushButton("Connect", buttonBox);
  new QWidget(buttonBox);
  QPushButton *disconnectButton = new QPushButton("Disconnect", buttonBox);
  new QWidget(buttonBox);
  QObject::connect(connectButton, SIGNAL(clicked()), this, SLOT(connectClicked()));
  QObject::connect(disconnectButton, SIGNAL(clicked()), this, SLOT(disconnectClicked()));
  QPushButton *showAllButton = new QPushButton("Show All Ports", buttonBox);
  new QWidget(buttonBox);
  QObject::connect(showAllButton, SIGNAL(clicked()), this, SLOT(showAllClicked()));
  QPushButton *hideAllButton = new QPushButton("Hide All Ports", buttonBox);
  new QWidget(buttonBox);
  QObject::connect(hideAllButton, SIGNAL(clicked()), this, SLOT(hideAllClicked()));
  QHBox *statusBox = new QHBox(this);
  statusBox->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  loadLabel = new QLabel(statusBox);
  new QWidget(statusBox);
  new QWidget(statusBox);
  rateLabel = new QLabel(statusBox);
  timer = new QTimer(this);
  connect(timer, SIGNAL(timeout()), this, SLOT(refreshGui()));
  timer->start(1000, false);
  connectorTimer = new QTimer(this);   
  connect(connectorTimer, SIGNAL(timeout()), this, SLOT(updateConnections()));
  ignoreChangeFlag = false;
  ignoreAnnounceFlag = false;
  updatePortLists();
  QObject::connect(readPorts, SIGNAL(expanded(QListViewItem *)), this, SLOT(listViewUpdate(QListViewItem *)));
  QObject::connect(writePorts, SIGNAL(expanded(QListViewItem *)), this, SLOT(listViewUpdate(QListViewItem *)));
  QObject::connect(readPorts, SIGNAL(collapsed(QListViewItem *)), this, SLOT(listViewUpdate(QListViewItem *)));
  QObject::connect(writePorts, SIGNAL(collapsed(QListViewItem *)), this, SLOT(listViewUpdate(QListViewItem *)));
}

void JackGui::displayAbout() {
 
    aboutWidget->about(this, "About QJackConnect", "QJackConnect 0.0.3b\n"
                     "by Matthias Nagorni\n"
                     "(c)2002 SuSE AG Nuremberg\n\n"
                     "QJackConnect is licensed under the GPL."
                     "\n\nAcknowledgements\n"
                     "----------------\n\n"  
                     "Kai Vehmanen has provided a patch to adapt\n"
                     "QJackConnect to the current CVS version of JACK.\n\n");
    aboutWidget->raise();
}

void JackGui::updatePortLists() {

  const char **readports, **writeports;
  int l1, l2, l3, readIndex, writeIndex;
  QString qs1, qs2;
  QStringList portStringList;
  bool portNew, deletePort, clientNew, deleteClient;
  JackClient *jackClient;
  JackPort *jackPort;

  if (ignoreChangeFlag) 
    return;
  readports = jack_get_ports(jack_handle, NULL, NULL, JackPortIsOutput);
  l1 = 0;
  while(readports && readports[l1]) {           // first pass: add new clients
    qs1 = QString(readports[l1]);
    qs2 = qs1.left(qs1.find(":"));
    clientNew = true;
    for (l2 = 0; l2 < readClientList.count(); l2++) {
      if (readClientList.at(l2)->clientName == qs2) {
        clientNew = false;
        break;
      }
    }
    if (clientNew) {          
      jackClient = new JackClient(qs2);
      PortItem *clientItem = new PortItem(readPorts, qs2);
//      clientItem->setSelectable(false);
      jackClient->listViewItem = clientItem;
      readClientList.append(jackClient);
    }
    l1++;
  }
  for (l2 = 0; l2 < readClientList.count(); l2++) {   // second pass: delete closed clients
    l1 = 0;
    deleteClient = true;
    while(readports && readports[l1]) {           
      qs1 = QString(readports[l1]);
      qs2 = qs1.left(qs1.find(":"));
      if (readClientList.at(l2)->clientName == qs2) { 
        deleteClient = false;
        break;
      }
      l1++;
    }  
    if (deleteClient && readClientList.count()) {
      delete readClientList.at(l2)->listViewItem;
      readClientList.remove(l2);
    }
  }
  l1 = 0;
  while(readports && readports[l1]) {           // third pass: add new ports
    qs1 = QString(readports[l1]);
    qs2 = qs1.left(qs1.find(":"));
    for (l2 = 0; l2 < readClientList.count(); l2++) {
      if (readClientList.at(l2)->clientName == qs2) {
        break;
      }
    }
    portNew = true;
    qs2 = qs1.right(qs1.length() - qs1.find(":") - 1);
    for (l3 = 0; l3 < readClientList.at(l2)->portList.count(); l3++) {
      if (readClientList.at(l2)->portList.at(l3)->portName == qs2) {
        portNew = false;
        break;
      }
    }  
    if (portNew) {    
      jackPort = new JackPort(qs2);
      jackPort->listViewItem = new PortItem(readClientList.at(l2)->listViewItem, qs2);
      readClientList.at(l2)->portList.append(jackPort);
    }
    l1++;
  }
  for (l2 = 0; l2 < readClientList.count(); l2++) {   // fourth pass: delete closed ports
    for (l3 = 0; l3 < readClientList.at(l2)->portList.count(); l3++) {
      l1 = 0;
      deletePort = true;
      while(readports && readports[l1]) {
        qs1 = QString(readports[l1]);
        qs2 = qs1.right(qs1.length() - qs1.find(":") - 1);
        if (readClientList.at(l2)->portList.at(l3)->portName == qs2) {
          deletePort = false;
          break;
        }
        l1++;
      }  
      if (deletePort && readClientList.at(l2)->portList.count()) {
        delete readClientList.at(l2)->portList.at(l3)->listViewItem;
        readClientList.at(l2)->portList.remove(l3);
      }
    }
  }  
//  writePorts->clear();
  writeports = jack_get_ports(jack_handle, NULL, NULL, JackPortIsInput);
  l1 = 0;
  while(writeports && writeports[l1]) {           // first pass: add new clients
    qs1 = QString(writeports[l1]);
    qs2 = qs1.left(qs1.find(":"));
    clientNew = true;
    for (l2 = 0; l2 < writeClientList.count(); l2++) {
      if (writeClientList.at(l2)->clientName == qs2) {
        clientNew = false;
        break;
      }
    }
    if (clientNew) {          
      jackClient = new JackClient(qs2);
      PortItem *clientItem = new PortItem(writePorts, qs2);
//      clientItem->setSelectable(false);
      jackClient->listViewItem = clientItem;
      writeClientList.append(jackClient);
    }
    l1++;
  }
  for (l2 = 0; l2 < writeClientList.count(); l2++) {   // second pass: delete closed clients
    l1 = 0;
    deleteClient = true;
    while(writeports && writeports[l1]) {           
      qs1 = QString(writeports[l1]);
      qs2 = qs1.left(qs1.find(":"));
      if (writeClientList.at(l2)->clientName == qs2) { 
        deleteClient = false;
        break;
      }
      l1++;
    }  
    if (deleteClient && writeClientList.count()) {
      delete writeClientList.at(l2)->listViewItem;
      writeClientList.remove(l2);
    }
  }
  l1 = 0;
  while(writeports && writeports[l1]) {           // third pass: add new ports
    qs1 = QString(writeports[l1]);
    qs2 = qs1.left(qs1.find(":"));
    for (l2 = 0; l2 < writeClientList.count(); l2++) {
      if (writeClientList.at(l2)->clientName == qs2) {
        break;
      }
    }
    portNew = true;
    qs2 = qs1.right(qs1.length() - qs1.find(":") - 1);
    for (l3 = 0; l3 < writeClientList.at(l2)->portList.count(); l3++) {
      if (writeClientList.at(l2)->portList.at(l3)->portName == qs2) {
        portNew = false;
        break;
      }
    }  
    if (portNew) {    
      jackPort = new JackPort(qs2);
      jackPort->listViewItem = new PortItem(writeClientList.at(l2)->listViewItem, qs2);
      writeClientList.at(l2)->portList.append(jackPort);
    }
    l1++;
  }
  for (l2 = 0; l2 < writeClientList.count(); l2++) {   // fourth pass: delete closed ports
    for (l3 = 0; l3 < writeClientList.at(l2)->portList.count(); l3++) {
      l1 = 0;
      deletePort = true;
      while(writeports && writeports[l1]) {
        qs1 = QString(writeports[l1]);
        qs2 = qs1.right(qs1.length() - qs1.find(":") - 1);
        if (writeClientList.at(l2)->portList.at(l3)->portName == qs2) {
          deletePort = false;
          break;
        }
        l1++;
      }  
      if (deletePort && writeClientList.at(l2)->portList.count()) {
        delete writeClientList.at(l2)->portList.at(l3)->listViewItem;
        writeClientList.at(l2)->portList.remove(l3);
      }
    }
  }  
}

void JackGui::updateConnections() {

  QString qs;
  int l1, l2, l3, l4, l5;

  ignoreChangeFlag = true;
  for (l1 = 0; l1 < readClientList.count(); l1++) {
    for (l2 = 0; l2 < readClientList.at(l1)->portList.count(); l2++) {
      const char **connections;
      readClientList.at(l1)->portList.at(l2)->connectedPorts.clear();
      qs = readClientList.at(l1)->clientName+":"+readClientList.at(l1)->portList.at(l2)->portName;
      connections = jack_port_get_all_connections(jack_handle,
                                                  jack_port_by_name(jack_handle, qs.latin1()));
      if (connections != NULL) {
        for (l3 = 0; connections[l3]; l3++) {
          for (l4 = 0; l4 < writeClientList.count(); l4++) {
            for (l5 = 0; l5 < writeClientList.at(l4)->portList.count(); l5++) {
              qs = writeClientList.at(l4)->clientName+":"+writeClientList.at(l4)->portList.at(l5)->portName;
              if (QString(connections[l3]) == qs) {
                readClientList.at(l1)->portList.at(l2)->connectedPorts.append(writeClientList.at(l4)->portList.at(l5));
              } 
            }
          }
        }
      }
      free(connections);
    }    
  }  
  connector->updateConnector();
  ignoreChangeFlag = false;
}

void JackGui::connectClicked() {

  QString readport, writeport;
  PortItem *readItem, *writeItem;
  int l1, readItemIndex, writeItemIndex, maxPortCount;
  bool readItemIsClient, writeItemIsClient;

  if ((readItem = (PortItem *)readPorts->selectedItem()) && (writeItem = (PortItem *)writePorts->selectedItem())) {
    readItemIsClient = false;
    for (l1 = 0; l1 < readClientList.count(); l1++) {
      if (readClientList.at(l1)->listViewItem == readItem) {
        readItemIsClient = true;
        readItemIndex = l1;
        break;
      }
    }
    writeItemIsClient = false;
    for (l1 = 0; l1 < writeClientList.count(); l1++) {
      if (writeClientList.at(l1)->listViewItem == writeItem) {
        writeItemIsClient = true;
        writeItemIndex = l1;
        break;
      }
    }
    if (readItemIsClient && writeItemIsClient) {
      maxPortCount = readClientList.at(readItemIndex)->portList.count();
      if (writeClientList.at(writeItemIndex)->portList.count() < maxPortCount) {
        maxPortCount = writeClientList.at(writeItemIndex)->portList.count();
      }
      for (l1 = 0; l1 < maxPortCount; l1++) {
        readport = readClientList.at(readItemIndex)->clientName+":"+readClientList.at(readItemIndex)->portList.at(l1)->portName;
        writeport = writeClientList.at(writeItemIndex)->clientName+":"+writeClientList.at(writeItemIndex)->portList.at(l1)->portName;
        if (!jack_connect(jack_handle, readport.latin1(),
                                       writeport.latin1())) {
        } else {
          printf("Could not connect %s to %s\n", readport.latin1(),  
                                                 writeport.latin1());
        }
      }
    } else {
      if (!readItemIsClient && !writeItemIsClient) {
        readport = readItem->parent()->text(0)+":"+readItem->text(0);
        writeport = writeItem->parent()->text(0)+":"+writeItem->text(0);
        if (!jack_connect(jack_handle, readport.latin1(),
                                       writeport.latin1())) {
        } else {
          printf("Could not connect %s to %s\n", readport.latin1(),
                                                 writeport.latin1());
        }
      }
    }
  }
}

void JackGui::disconnectClicked() {

  QString readport, writeport;
  PortItem *readItem, *writeItem;
  int l1, readItemIndex, writeItemIndex, maxPortCount;
  bool readItemIsClient, writeItemIsClient;  

  if ((readItem = (PortItem *)readPorts->selectedItem()) && (writeItem = (PortItem *)writePorts->selectedItem())) {
    readItemIsClient = false;
    for (l1 = 0; l1 < readClientList.count(); l1++) {
      if (readClientList.at(l1)->listViewItem == readItem) {
        readItemIsClient = true;
        readItemIndex = l1;
        break;
      }
    }
    writeItemIsClient = false;
    for (l1 = 0; l1 < writeClientList.count(); l1++) {
      if (writeClientList.at(l1)->listViewItem == writeItem) {
        writeItemIsClient = true;
        writeItemIndex = l1;
        break;
      }
    }
    if (readItemIsClient && writeItemIsClient) {
      maxPortCount = readClientList.at(readItemIndex)->portList.count();
      if (writeClientList.at(writeItemIndex)->portList.count() < maxPortCount) {
        maxPortCount = writeClientList.at(writeItemIndex)->portList.count();
      }
      for (l1 = 0; l1 < maxPortCount; l1++) {
        readport = readClientList.at(readItemIndex)->clientName+":"+readClientList.at(readItemIndex)->portList.at(l1)->portName;
        writeport = writeClientList.at(writeItemIndex)->clientName+":"+writeClientList.at(writeItemIndex)->portList.at(l1)->portName;
        if (!jack_disconnect(jack_handle, readport.latin1(),
                                       writeport.latin1())) {
        } else {
          printf("Could not disconnect %s from %s\n", readport.latin1(),  
                                                 writeport.latin1());
        }
      }
    } else {
      if (!readItemIsClient && !writeItemIsClient) {
        readport = readItem->parent()->text(0)+":"+readItem->text(0);
        writeport = writeItem->parent()->text(0)+":"+writeItem->text(0);
        if (!jack_disconnect(jack_handle, readport.latin1(),
                                       writeport.latin1())) {
        } else {
          printf("Could not disconnect %s from %s\n", readport.latin1(),
                                                 writeport.latin1());
        }
      }
    }
  }
}

void JackGui::refreshGui() {

  QString qs;

  rate = jack_get_sample_rate(jack_handle);
  cpuload = jack_cpu_load(jack_handle);
  qs.sprintf("  CPU load: %5.2f", cpuload);
  loadLabel->setText(qs);
  qs.sprintf("Sample rate: %d", rate);  
  rateLabel->setText(qs);
}

void JackGui::closeJack() {

  fprintf(stderr, "\n\nClosing jack client.\n\n");
  jack_deactivate(jack_handle);
  jack_client_close(jack_handle);
}

void JackGui::jackAnnounce(int fd) {

  char dummy;

  if (ignoreAnnounceFlag) 
    return;
  read(fd, &dummy, 1);
  ignoreAnnounceFlag = true;
  updatePortLists();
  connectorTimer->start(200, true);
  ignoreAnnounceFlag = false;
}

void JackGui::listViewUpdate(QListViewItem *item) {

  connectorTimer->start(200, true);
}

void JackGui::showAllClicked() {

  int l1;

  for (l1 = 0; l1 < readClientList.count(); l1++) {
    readPorts->setOpen(readClientList.at(l1)->listViewItem, true);
  }
  for (l1 = 0; l1 < writeClientList.count(); l1++) {
    writePorts->setOpen(writeClientList.at(l1)->listViewItem, true);
  }
}

void JackGui::hideAllClicked() {

  int l1;

  for (l1 = 0; l1 < readClientList.count(); l1++) {
    readPorts->setOpen(readClientList.at(l1)->listViewItem, false);
  }
  for (l1 = 0; l1 < writeClientList.count(); l1++) {
    writePorts->setOpen(writeClientList.at(l1)->listViewItem, false);
  }
}
