#include <stdio.h>      
#include <stdlib.h>     
#include <netinet/in.h>
#include <netdb.h> 
#include <unistd.h>
#include <sys/socket.h>   
#include <getopt.h>  
#include <qapplication.h>
#include <qmainwindow.h>
#include <qmenubar.h>   
#include <qpopupmenu.h>
#include <qvbox.h>
#include <qscrollview.h>
#include <jack/jack.h>
#include "jackgui.h"

int jackAnnounceFd, callbackFd;

static struct option options[] =
        {{"help", 0, 0, 'h'},
         {0, 0, 0, 0}};

void portRegistrationCallback(jack_port_id_t port_id, int n, void *arg) {

  char dummy;

  write(callbackFd, &dummy, 1);
}

int graphOrderCallBack(void *arg) {

  char dummy;

  write(callbackFd, &dummy, 1);
  return(0);
}

int main(int argc, char *argv[])  
{
  QString aboutText ("QJackConnect 0.0.3b\n"
                     "by Matthias Nagorni\n"
                     "(c)2002 SuSE AG Nuremberg\n\n"
                     "QJackConnect is licensed under the GPL."
                     "\n\nAcknowledgements\n"
                     "----------------\n\n"
                     "Kai Vehmanen has provided a patch to adapt\n"
                     "QJackConnect to the current CVS version of JACK.\n\n");
  QApplication *qApp = new QApplication(argc, argv);
  QMainWindow *top = new QMainWindow();
  top->setCaption("QJackConnect");
  jack_client_t *jack_handle;
  int getopt_return;
  int option_index; 
  int pipeFd[2];
  while((getopt_return = getopt_long(argc, argv, "h", options, &option_index)) >= 0) {
    switch(getopt_return) {
    case 'h':
        printf("\n%s", aboutText.latin1());
        exit(EXIT_SUCCESS);
        break;
    }
  }
  if (pipe(pipeFd) < 0) {
    fprintf(stderr, "Error: Could not create pipe.\n");
  }
  callbackFd = pipeFd[1];
  jackAnnounceFd = pipeFd[0];
  if ((jack_handle = jack_client_new("QJackConnect")) == 0) {
    fprintf(stderr, "Could not connect to jack server.\n");
    exit(1);
  }
  jack_set_graph_order_callback(jack_handle, graphOrderCallBack, 0);
  jack_set_port_registration_callback(jack_handle, portRegistrationCallback, 0);
  JackGui *jackGui = new JackGui(jack_handle, top); 
  QSocketNotifier *jackNotifier = new QSocketNotifier(jackAnnounceFd, QSocketNotifier::Read);
  QObject::connect(jackNotifier, SIGNAL(activated(int)),
                   jackGui, SLOT(jackAnnounce(int)));
  QPopupMenu *filePopup = new QPopupMenu(top); 
  QPopupMenu *aboutMenu = new QPopupMenu(top);
  top->menuBar()->insertItem("&File", filePopup);
  top->menuBar()->insertSeparator(); top->menuBar()->insertItem("&About", aboutMenu); 
  filePopup->insertItem("&Quit", qApp, SLOT(quit()));
  QObject::connect(qApp, SIGNAL(aboutToQuit()), jackGui, SLOT(closeJack()));
  aboutMenu->insertItem("About QJackConnect", jackGui, SLOT(displayAbout())); 
  top->setGeometry(0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT); 
  top->setCentralWidget(jackGui); 
  top->show();
  jack_activate(jack_handle);
  qApp->setMainWidget(top); 
  return qApp->exec(); 
}
